@@ -27,14 +27,6 @@ popd
 ./cryptofuzz/generate_dict
 
 # Run Cryptofuzz.
-./cryptofuzz/cryptofuzz -dict="cryptofuzz-dict.txt" "nss/fuzz/corpus/cryptofuzz" "$@"
-
-# Alert if version is older than half a year.
-cryptofuzz_timestamp=$(git -C cryptofuzz show -s --format=%ct $CRYPTOFUZZ_VERSION)
-current_timestamp=$(date +%s)
-half_a_year=$((60 * 60 * 24 * 183))
-
-if [ $((current_timestamp - cryptofuzz_timestamp)) -gt $half_a_year ]; then
-    echo "Cryptofuzz version is older than half a year. Please consider updating it (and Botan). Thanks!" >&2
-    exit 1
-fi
+# Decrease the default ASAN quarantine size of 256 MB as we tend to run
+# out of memory on 32-bit.
+ASAN_OPTIONS="quarantine_size_mb=64" ./cryptofuzz/cryptofuzz -dict="cryptofuzz-dict.txt" --force-module=nss "nss/fuzz/corpus/cryptofuzz" "$@"
