@@ -446,6 +446,9 @@ static const struct FREEBLVectorStr vector = {
 
     X25519_DerivePublicKey,
     /* End of version 3.029 */
+
+    EC_DerivePublicKey,
+    /* End of version 3.030 */
 };
 
 const FREEBLVector*
