@@ -19,6 +19,8 @@
 #include "secpkcs5.h"
 #include "secerr.h"
 
+#include <limits.h>
+
 /*
  * -------------------------------------------------------------------
  * Cipher stuff.
@@ -32,6 +34,27 @@ typedef SECStatus (*sec_pkcs7_cipher_function)(void *,
                                                unsigned int);
 typedef SECStatus (*sec_pkcs7_cipher_destroy)(void *, PRBool);
 
+static SECStatus
+SECPKCS7Cipher_PK11_CipherOp(void *vctx, unsigned char *output,
+                             unsigned int *outputLen, unsigned int maxOutputLen,
+                             const unsigned char *input, unsigned int inputLen)
+{
+    PK11Context *ctx = vctx;
+    PORT_Assert(maxOutputLen <= INT_MAX);
+    int signedOutputLen = maxOutputLen;
+    SECStatus rv = PK11_CipherOp(ctx, output, &signedOutputLen, maxOutputLen, input, inputLen);
+    PORT_Assert(signedOutputLen >= 0);
+    *outputLen = signedOutputLen;
+    return rv;
+}
+static SECStatus
+SECPKCS7Cipher_PK11_DestroyContext(void *vctx, PRBool freeit)
+{
+    PK11Context *ctx = vctx;
+    PK11_DestroyContext(ctx, freeit);
+    return SECSuccess;
+}
+
 #define BLOCK_SIZE 4096
 
 struct sec_pkcs7_cipher_object {
@@ -118,8 +141,8 @@ sec_PKCS7CreateDecryptObject(PK11SymKey *key, SECAlgorithmID *algid)
     }
 
     result->cx = ciphercx;
-    result->doit = (sec_pkcs7_cipher_function)PK11_CipherOp;
-    result->destroy = (sec_pkcs7_cipher_destroy)PK11_DestroyContext;
+    result->doit = SECPKCS7Cipher_PK11_CipherOp;
+    result->destroy = SECPKCS7Cipher_PK11_DestroyContext;
     result->encrypt = PR_FALSE;
     result->pending_count = 0;
 
@@ -211,8 +234,8 @@ sec_PKCS7CreateEncryptObject(PLArenaPool *poolp, PK11SymKey *key,
     SECITEM_FreeItem(param, PR_TRUE);
 
     result->cx = ciphercx;
-    result->doit = (sec_pkcs7_cipher_function)PK11_CipherOp;
-    result->destroy = (sec_pkcs7_cipher_destroy)PK11_DestroyContext;
+    result->doit = SECPKCS7Cipher_PK11_CipherOp;
+    result->destroy = SECPKCS7Cipher_PK11_DestroyContext;
     result->encrypt = PR_TRUE;
     result->pending_count = 0;
 
@@ -458,7 +481,6 @@ sec_PKCS7Decrypt(sec_PKCS7CipherObject *obj, unsigned char *output,
          * If we do not, there is something wrong, either with our own
          * logic or with (length of) the data given to us.
          */
-        PORT_Assert((padsize == 0) || (pcount % padsize) == 0);
         if ((padsize != 0) && (pcount % padsize) != 0) {
             PORT_Assert(final);
             PORT_SetError(SEC_ERROR_BAD_DATA);
